@@ -13,17 +13,20 @@ utils::globalVariables(c("vecg","nComponents","modellags","phiEstimate","y","dat
 #' Function estimates ETS in a form of the Single Source of Error State-space
 #' model of the following type:
 #'
-#' \eqn{y_[t] = o_[t] (w(v_[t-l]) + x_t a_[t-1] + r(v_[t-l]) \epsilon_[t])}
+#' \deqn{y_{t} = o_{t} (w(v_{t-l}) + x_t a_{t-1} + r(v_{t-l}) \epsilon_{t})}
 #'
-#' \eqn{v_[t] = f(v_[t-l]) + g(v_[t-l]) \epsilon_[t]}
+#' \deqn{v_{t} = f(v_{t-l}) + g(v_{t-l}) \epsilon_{t}}
 #'
-#' \eqn{a_[t] = F_[X] a_[t-1] + g_[X] \epsilon_[t] / x_[t]}
+#' \deqn{a_{t} = F_{X} a_{t-1} + g_{X} \epsilon_{t} / x_{t}}
 #'
-#' Where \eqn{o_[t]} is Bernoulli distributed random variable (in case of
-#' normal data it equals to 1 for all observations), \eqn{v_[t]} is a state
-#' vector and \eqn{l} is a vector of lags, \eqn{x_t} vector of exogenous
-#' parameters. w(.) is measurement function, r(.) is an error function, f(.) is
-#' transition function and g(.) is persistence function.
+#' Where \eqn{o_{t}} is the Bernoulli distributed random variable (in case of
+#' normal data it equals to 1 for all observations), \eqn{v_{t}} is the state
+#' vector and \eqn{l} is the vector of lags, \eqn{x_t} is the vector of
+#' exogenous variables. w(.) is the measurement function, r(.) is the error
+#' function, f(.) is the transition function and g(.) is the persistence
+#' function. \eqn{a_t} is the vector of parameters for exogenous variables,
+#' \eqn{F_{X}} is the \code{transitionX} matrix and \eqn{g_{X}} is the
+#' \code{persistenceX} matrix.  Finally, \eqn{\epsilon_{t}} is the error term.
 #'
 #' For the details see Hyndman et al.(2008).
 #'
@@ -114,6 +117,7 @@ utils::globalVariables(c("vecg","nComponents","modellags","phiEstimate","y","dat
 #' \item \code{s2} - variance of the residuals (taking degrees of freedom into account).
 #' \item \code{intervals} - type of intervals asked by user.
 #' \item \code{level} - confidence level for intervals.
+#' \item \code{cumulative} - whether the produced forecast was cumulative or not.
 #' \item \code{actuals} - original data.
 #' \item \code{holdout} - holdout part of the original data.
 #' \item \code{iprob} - fitted and forecasted values of the probability of demand occurrence.
@@ -154,6 +158,7 @@ utils::globalVariables(c("vecg","nComponents","modellags","phiEstimate","y","dat
 #' \item \code{s2} - variance of additive error of combined one-step-ahead forecasts,
 #' \item \code{intervals},
 #' \item \code{level},
+#' \item \code{cumulative},
 #' \item \code{actuals},
 #' \item \code{holdout},
 #' \item \code{iprob},
@@ -224,8 +229,8 @@ utils::globalVariables(c("vecg","nComponents","modellags","phiEstimate","y","dat
 #' @export es
 es <- function(data, model="ZZZ", persistence=NULL, phi=NULL,
                initial=c("optimal","backcasting"), initialSeason=NULL, ic=c("AICc","AIC","BIC"),
-               cfType=c("MSE","MAE","HAM","MLSTFE","MSTFE","MSEh"),
-               h=10, holdout=FALSE,
+               cfType=c("MSE","MAE","HAM","GMSTFE","MSTFE","MSEh","TFL"),
+               h=10, holdout=FALSE, cumulative=FALSE,
                intervals=c("none","parametric","semiparametric","nonparametric"), level=0.95,
                intermittent=c("none","auto","fixed","croston","tsb","sba"),
                bounds=c("usual","admissible","none"),
@@ -1066,7 +1071,7 @@ CreatorES <- function(silent=FALSE,...){
     }
     else{
         # smoothingParameters <- cbind(c(0.2,0.1,0.05),rep(0.05,3));
-        smoothingParameters <- cbind(c(0.3,0.2,0.1),c(0.3,0.2,0.1));
+        smoothingParameters <- cbind(c(0.3,0.2,0.1),c(0.1,0.05,0.01));
     }
 
 ##### Preset y.fit, y.for, errors and basic parameters #####
@@ -1242,20 +1247,26 @@ CreatorES <- function(silent=FALSE,...){
 
             # Number of parameters
             nParam <- nComponents + damped + (nComponents + (maxlag-1) * (Stype!="N")) * (initialType!="b") + !is.null(xreg) * nExovars + (updateX)*(nExovars^2 + nExovars);
-            if(nParam!=length(providedC)){
-                warning(paste0("Number of parameters to optimise differes from the length of C:",nParam," vs ",length(providedC),".\n",
-                               "We will have to drop parameter C."),call.=FALSE);
-                providedC <- NULL;
+            if(!is.null(providedC)){
+                if(nParam!=length(providedC)){
+                    warning(paste0("Number of parameters to optimise differes from the length of C:",nParam," vs ",length(providedC),".\n",
+                                   "We will have to drop parameter C."),call.=FALSE);
+                    providedC <- NULL;
+                }
             }
-            if(nParam!=length(providedCLower)){
-                warning(paste0("Number of parameters to optimise differes from the length of CLower:",nParam," vs ",length(providedCLower),".\n",
-                               "We will have to drop parameter CLower."),call.=FALSE);
-                providedCLower <- NULL;
+            if(!is.null(providedCLower)){
+                if(nParam!=length(providedCLower)){
+                    warning(paste0("Number of parameters to optimise differes from the length of CLower:",nParam," vs ",length(providedCLower),".\n",
+                                   "We will have to drop parameter CLower."),call.=FALSE);
+                    providedCLower <- NULL;
+                }
             }
-            if(nParam!=length(providedCUpper)){
-                warning(paste0("Number of parameters to optimise differes from the length of CUpper:",nParam," vs ",length(providedCUpper),".\n",
-                               "We will have to drop parameter CUpper."),call.=FALSE);
-                providedCUpper <- NULL;
+            if(!is.null(providedCUpper)){
+                if(nParam!=length(providedCUpper)){
+                    warning(paste0("Number of parameters to optimise differes from the length of CUpper:",nParam," vs ",length(providedCUpper),".\n",
+                                   "We will have to drop parameter CUpper."),call.=FALSE);
+                    providedCUpper <- NULL;
+                }
             }
             C <- providedC;
             CLower <- providedCLower;
@@ -1279,6 +1290,11 @@ CreatorES <- function(silent=FALSE,...){
             CLower <- NULL;
             CUpper <- NULL;
         }
+
+        # If we need to estimate phi, make it NULL, so the next maker works
+        if(phiEstimate){
+            phi <- NULL
+        }
     }
 
 ##### Now do estimation and model selection #####
@@ -1567,16 +1583,16 @@ CreatorES <- function(silent=FALSE,...){
         forecasts.list <- forecasts.list[,!badStuff];
         icWeights <- icWeights[!badStuff];
         model.current <- model.current[!badStuff];
-        y.fit <- ts(fitted.list %*% icWeights,start=start(data),frequency=frequency(data));
-        y.for <- ts(forecasts.list %*% icWeights,start=time(data)[obsInsample]+deltat(data),frequency=frequency(data));
-        errors <- ts(c(y) - y.fit,start=start(data),frequency=frequency(data));
+        y.fit <- ts(fitted.list %*% icWeights,start=start(data),frequency=datafreq);
+        y.for <- ts(forecasts.list %*% icWeights,start=time(data)[obsInsample]+deltat(data),frequency=datafreq);
+        errors <- ts(c(y) - y.fit,start=start(data),frequency=datafreq);
         s2 <- mean(errors^2);
         names(icWeights) <- model.current;
         if(intervals){
             lowerList <- lowerList[,!badStuff];
             upperList <- upperList[,!badStuff];
-            y.low <- ts(lowerList %*% icWeights,start=start(y.for),frequency=frequency(data));
-            y.high <- ts(upperList %*% icWeights,start=start(y.for),frequency=frequency(data));
+            y.low <- ts(lowerList %*% icWeights,start=start(y.for),frequency=datafreq);
+            y.high <- ts(upperList %*% icWeights,start=start(y.for),frequency=datafreq);
         }
         else{
             y.low <- NA;
@@ -1618,8 +1634,32 @@ CreatorES <- function(silent=FALSE,...){
 
 ##### Now let's deal with holdout #####
     if(holdout){
-        y.holdout <- ts(data[(obsInsample+1):obsAll],start=start(y.for),frequency=frequency(data));
-        errormeasures <- errorMeasurer(y.holdout,y.for,y);
+        y.holdout <- ts(data[(obsInsample+1):obsAll],start=start(y.for),frequency=datafreq);
+        if(cumulative){
+            errormeasures <- errorMeasurer(sum(y.holdout),y.for,h*y);
+        }
+        else{
+            errormeasures <- errorMeasurer(y.holdout,y.for,y);
+        }
+
+
+# Add PLS
+        errormeasuresNames <- names(errormeasures);
+        if(all(intermittent!=c("n","none"))){
+            errormeasures <- c(errormeasures, suppressWarnings(pls(actuals=y.holdout, forecasts=y.for, Etype=Etype,
+                                                                   sigma=s2, trace=FALSE, iprob=pt[obsInsample+c(1:h)])));
+        }
+        else{
+            if(multisteps){
+                sigma <- t(errors.mat) %*% errors.mat / obsInsample;
+            }
+            else{
+                sigma <- s2;
+            }
+            errormeasures <- c(errormeasures, suppressWarnings(pls(actuals=y.holdout, forecasts=y.for, Etype=Etype,
+                                                                   sigma=sigma, trace=multisteps, iprob=pt[obsInsample+c(1:h)])));
+        }
+        names(errormeasures) <- c(errormeasuresNames,"PLS");
     }
     else{
         y.holdout <- NA;
@@ -1647,24 +1687,35 @@ CreatorES <- function(silent=FALSE,...){
 
 ##### Make a plot #####
     if(!silentGraph){
+        y.for.new <- y.for;
+        y.high.new <- y.high;
+        y.low.new <- y.low;
+        if(cumulative){
+            y.for.new <- ts(rep(y.for/h,h),start=start(y.for),frequency=datafreq)
+            if(intervals){
+                y.high.new <- ts(rep(y.high/h,h),start=start(y.for),frequency=datafreq)
+                y.low.new <- ts(rep(y.low/h,h),start=start(y.for),frequency=datafreq)
+            }
+        }
+
         if(intervals){
-            graphmaker(actuals=data,forecast=y.for,fitted=y.fit, lower=y.low,upper=y.high,
-                       level=level,legend=!silentLegend,main=modelname);
+            graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit, lower=y.low.new,upper=y.high.new,
+                       level=level,legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
         else{
-            graphmaker(actuals=data,forecast=y.for,fitted=y.fit,
-                    level=level,legend=!silentLegend,main=modelname);
+            graphmaker(actuals=data,forecast=y.for.new,fitted=y.fit,
+                       level=level,legend=!silentLegend,main=modelname,cumulative=cumulative);
         }
     }
 
-    ##### Return values #####
+##### Return values #####
     if(modelDo!="combine"){
         model <- list(model=modelname,formula=esFormula,timeElapsed=Sys.time()-startTime,
                       states=matvt,persistence=persistence,phi=phi,
                       initialType=initialType,initial=initialValue,initialSeason=initialSeason,
                       nParam=nParam,
                       fitted=y.fit,forecast=y.for,lower=y.low,upper=y.high,residuals=errors,
-                      errors=errors.mat,s2=s2,intervals=intervalsType,level=level,
+                      errors=errors.mat,s2=s2,intervals=intervalsType,level=level,cumulative=cumulative,
                       actuals=data,holdout=y.holdout,iprob=pt,intermittent=intermittent,
                       xreg=xreg,updateX=updateX,initialX=initialX,persistenceX=vecgX,transitionX=matFX,
                       ICs=ICs,logLik=logLik,cf=cfObjective,cfType=cfType,FI=FI,accuracy=errormeasures);
@@ -1675,6 +1726,7 @@ CreatorES <- function(silent=FALSE,...){
                       initialType=initialType,
                       fitted=y.fit,forecast=y.for,
                       lower=y.low,upper=y.high,residuals=errors,s2=s2,intervals=intervalsType,level=level,
+                      cumulative=cumulative,
                       actuals=data,holdout=y.holdout,iprob=pt,intermittent=intermittent,
                       xreg=xreg,updateX=updateX,
                       ICs=ICs,ICw=icWeights,cf=NULL,cfType=cfType,accuracy=errormeasures);
